using System;
using System.Collections;

namespace Criptologia.LFSR {
   
   public class Principal {
      
      public static void Main() {
         
         string input = Console.ReadLine();
         int[] inputFormatted = new int[input.Length];
         char[] inputChar = input.ToCharArray();
         bool cap_1 = true;
         
         // Transformem les dades d'un string a un vector d'enters i
         // comprovem que les dades siguin correctes (noms pot contenir
         // 0's i 1's i no pot ser tot 0's)
         for (int i=0; i<inputChar.Length; i++){
            if (inputChar[i] == '0') {
              inputFormatted[i] = 0;
            } else if (inputChar[i] == '1') {
              cap_1 = false;
              inputFormatted[i] = 1;
            } else {
               Console.WriteLine("Dades introduides no vlides!");
               return;  // Error
            }
         }
         
         if (cap_1) {
            Console.WriteLine("Dades introduides no vlides: no hi ha cap 1!");
            return;  // Error, no hi ha cap 1 a l'entrada
         }
         
         BerlekampMassey algorisme = new BerlekampMassey(inputFormatted);
         algorisme.Executar();
         algorisme.printResultats();
      }
   }
   
   // Algorisme Berlekamp-Massey
   // Troba la configuraci inicial de celles d'un LFSR a partir d'una
   // seqncia de sortida prou gran.
   // * LFSR = Linear Feedback Shift Register
   public class BerlekampMassey {
      private int N; // Tamany de l'entrada
      private int[] a;  // a(n): element n de l'entrada, n=1..N
      private int[] L;  // L(n): Longitud actual del LFSR(n)
      private int[] d;  // d(n): Diferencia a(n+1) i l'element (n+1) de sortida de l'actual LFSR(n)
      private int[] k;  // k(n): punt a on el canvi ms recent de longitud ha ocurregut abans d'n
      private ArrayList[] sigma; // sigma(i, j): coeficient j del polinomi de connexi i
      private int[] deltaL; // delta(n): n+1 - 2*L(n)
      private int n; // Iteraci del proces

      // Inicilialitzacions i primers passos
      public BerlekampMassey (int[] input) {
         N = input.Length;

         // Copiem l'entrada a un nou vector de forma que la primera posici estigui
         // a la posici 1 i no a la 0
         a = new int[N+1];
         for (int i=0; i<N; i++) {
            a[i+1] = input[i];
         }
         L = new int[N+1];
         d = new int[N+1];
         k = new int[N+1];
         deltaL = new int[N+1];
         sigma = new ArrayList[N+1];

         
         // Condicions inicials
         n=1;
         d[0] = a[1];
         L[0] = 0;
         k[0] = 0;
         ///sigma[0,0] = 1;
         sigma[0] = new ArrayList();
         sigma[0].Add(1);
         
         // Si la primera posici de l'entrada es 1
         if (a[1] != 0) {
            //   L(0) = 0, L(1) = 1,
            //   k(1) = 0, d(0) = a1,
            //   sigma(0)(x) = 1, sigma(1)(x) = 1 + x
            L[1] = 1;
            k[1] = 0;
            ///sigma[1,0] = 1;
            ///sigma[1,1] = 1;
            sigma[1] = new ArrayList();
            sigma[1].Add(1);
            sigma[1].Add(1);
         } else { //   If a1 = ... = am-1 = 0 and am != 0
            // Hi han ceros inicials que podem saltar rapid
            //   L(0) = ... L(m-1) = 0, L(m) = m,
            //   k(m) = m-1,
            //   d(m-1) = am,
            //   sigma(0)(x) = ... sigma(m-1)(x) = 1, sigma(m)(x) = 1 + xm
            int i;
            for(i=1; i<=N; i++) {
               if (a[i] == 0) {
                  L[i] = 0;
                  ///sigma[i,0] = 1;
                  sigma[i] = new ArrayList();
                  sigma[i].Add(1);
               } else {
                  break;
               }
            }
            
            n = i;
            if (n<=N) {
               L[n] = n;
               k[n] = n-1;
               d[n-1] = a[n];
               ///sigma[n,0] = 1;
               sigma[n] = new ArrayList();
               sigma[n].Add(1);
               
               for(i=1;i<n;i++) {
                  ///sigma[n,i] = 0;
                  sigma[n].Add(0);
               }
               ///sigma[n,n] = 1;
               sigma[n].Add(1);
            }
         }
      }

      // Executar tot el processament que resti per fer
      public void Executar(){
         while (n<N) {
            Step();
            n++;
         }
         int j = 0;
      }
      
      private void Step() {
         //   d(n) = an+1 + sigma1(n) * an+ ... + sigmaL(n)(n) * an+1-L(n)
         d[n] = (a[n+1] + sumarCoeficients(n)) % 2;
         
         //   If d(n) = 0
         if (d[n] == 0) {
            //   L(n+1) = L(n), L(1) = 1,
            //   k(n+1) = k(n),
            //   sigma(n+1)(x) = sigma(n)(x) 
            L[n+1] = L[n];
            k[n+1] = k[n];
            sigma[n+1] = (ArrayList)sigma[n].Clone();
         } else {    //   If d(n) != 0
            //    deltaL = n + 1 - 2*L(n)
            deltaL[n+1] = n+1 - 2*L[n];
            
            //    If deltaL > 0
            if (deltaL[n+1] > 0){
               //   L(n+1) = L(n) + deltaL,
               //   k(n+1) = n 
               L[n+1] = L[n] + deltaL[n+1];
               k[n+1] = n;
            } else  {   //    If deltaL <= 0
               //   L(n+1) = L(n),
               //   k(n+1) = k(n) 
               L[n+1] = L[n];
               k[n+1] = k[n];
            }
            //    sigma(n+1)(x) = sigma(n)(x) - (d(n)/d(k(n))) * x(n-k(n)) * sigma(k(n))(x) 
            multiplicarPolinomy(n);
         }
      }
      
      private int sumarCoeficients(int num) {
         int result = 0;
         
         int i = 0;
         foreach(int valor in sigma[num]) {
            if (i==0){  // No es considera el primer element
               i++;
               continue;
            }
            if (i <= L[num]) {
               result += valor * a[num+1-i];
               i++;
            } else {
               break;
            }
         }
         result %= 2;
         return result;
      }
      
      // sigma(n+1)(x) = sigma(n)(x) - (d(n)/d(k(n))) * x(n-k(n)) * sigma(k(n))(x)
      private void multiplicarPolinomy(int n){
         //1.) Comprovar si d[n]/d[k[n]] s 1
         // d[n] s 1, de lo contrari no es cridaria aquesta funci
         // d[k[n]] sempre s 1
         // Per tant d[n]/d[k[n]] = 1
         
         //2.) Multiplicar: x^(n-k[n]) * sigma[k[n]]
         // Aix s igual a desplazar sigma(k[n]) a la dreta n-k[n] posicions
         
         int intDiff = n - k[n];
         
         ArrayList arrProduct = new ArrayList();
         
         for(int i=0;i<intDiff; i++) {
            ///arrProduct[i] = 0;
            arrProduct.Add(0);
         }
         
         foreach (int valor in sigma[k[n]]) {
            arrProduct.Add(valor);
         }
         
         //3.) Restem arrProcut a sigma[n]
         // Utilitzem la suma en comptes de la resta per evitar problemes de signe
         // tenint en compte que degut a les operacions modul 2 restar es igual a sumar
         int intLengthA = sigma[n].Count;
         int intLengthB = arrProduct.Count;
         
         for(int i=0; i < Math.Max(intLengthA, intLengthB); i++) {
            if(i >= intLengthA){
               break;
            } else {
               if(i >= intLengthB) {
                  arrProduct.Add(ArrayListPos(sigma[n], i));
               } else {
                  int calc = (ArrayListPos(arrProduct, i) + ArrayListPos(sigma[n], i) ) % 2;
                  arrProduct.RemoveAt(i);
                  arrProduct.Insert(i, calc);
                  
                  
               }
            }
         }
         sigma[n+1] = (ArrayList)arrProduct.Clone();
      }
      
      // Retorna l'element en la posici indicada d'un ArrayList
      private int ArrayListPos(ArrayList lista, int pos) {
         int i = 0;
         foreach(int obj in lista) {
            if (pos == i) {
               return obj;
            }
            i++;
         }
         return -1;  // Error
      }
      
      public void printResultats() {
         int j=0;
         
         // Taula de l'evoluci del clcul
         Console.WriteLine("n\t a[n]\t d[n]\t L[n]\t k[n]\t deltaL[n]\t sigma[n]");
         for (int i=0; i<=n;i++) {
            Console.Write("{0}\t {1}\t {2}\t {3}\t {4}\t {5}\t\t ", i, a[i], d[i], L[i], k[i], deltaL[i]);
            // Mostrem el polinomi de conexi de cada nivell
            j = 0;
            foreach (int valor in sigma[i]) {
               if (j==0) {
                  Console.Write(valor);
               } else {
                  if (valor==1){
                     Console.Write(" + X^"+j);
                  } else if (valor!=0) {
                     Console.Write(" + "+valor+"X^"+j);
                  }
               }
               j++;
            }
            Console.WriteLine("");
         }
         // Connexions XOR
         Console.WriteLine("Hi ha "+L[n]+ " celles, numerades de 1 a " +L[n]+ ".");
         j=0;
         foreach (int valor in sigma[n]) {
            // La primera i ultima posici del polinomi no s'han de considerar
            if ((j!=0) && (j!=sigma[n].Count-1)){
               if (valor!=0){
                  Console.WriteLine("XOR a la posici "+j);
               }
            }
            j++;
         }
         
         // Estat inicial de les celles
         for(int r=0;r<L[n];r++) {
            Console.Write("["+a[ L[n]-r ]+"]");
         }
         Console.WriteLine("->output");
      }
   }
}

